from Environment import Environment
from jivi.fs import *

tables = File.jread('out.json')
tt = time()




alias_table = {}
bad = {}

for tablename in tables:
	tmp = tablename[0]
	for i in range(1,len(tablename)):
		tmp += tablename[i]
		
		if tmp in alias_table:
			bad[tmp] = 1
			del alias_table[tmp]
		if tmp in bad: continue
		alias_table[tmp] = 1
		


print(time() - tt)


def test():
	tablename = "ordrln"
	for i in range(len(tablename) - 1,1,-1):
		print(tablename[:i])
		
	
	


exit()
shortcut_link = r"C:\Users\jorisv\Desktop\Intex_links\local\MEC.lnk"

x = Environment(shortcut_link)


from jivi.oe.Statement.Define.Define import Define
class Widget(Define):
	type = 'widget'
	def __init__(self,*a,**b):
		Define.__init__(self,*a,**b)

	def pre_init(self):
		Define.pre_init(self)
		
	def set_variable(self):
		if not self.read_input('datatype','as'):
			if self.read_input('like','like'):
				likeel = self.window.get_widget(self.like)
				if likeel:
					for a,b in likeel.tab.items():
						self.set_if_none(a,b)
	

		


from jivi.oe.Statement.Define import *

from jivi.fs import *
exit()

ar = []
for fp in Dir.files('.',ex='py'):
	n = File.name(fp)
	if n in ['k','__init__']: continue
	ar.append("""from jivi.oe.Statement.Define.{n} import {n}""".format(n=n))
	
	
print("\n".join(ar))


from jivi.oe.constants import *
from jivi.fs import *

def remove_from_list(ar,to_remove):
	for i in sorted(to_remove,reverse=1):
		ar = ar[:i] + ar[i+1:]
	return ar



		


def create_tab(ar,t):
	tab = {}
	for a in ar:
		tab[a] = t[a] if a in t else ''
	return tab

	





class StatementReader:
		
	@staticmethod
	def assign(a):
		t = {}
		n = 0
		while (n + 2) < len(a):
			t[a[n]] = a[n+2]
			n += 3
		return t

class ArHelper:

		
	@staticmethod
	def same_rest(a,b):
		try:
			for i,x in enumerate(b):
				if a[i] != x: return 0
			return a[len(b):]
		except:
			return 0
	@staticmethod
	def same(a,b):
		try:
			for i,x in enumerate(b):
				if a[i] != x: return 0
			return 1
		except:
			return 0
			
	@staticmethod
	def has(a,b):
		for i,x in enumerate(a):
			if x == b[0]:
				if same(a[i:],b):
					return 1
					
	@staticmethod
	def strip_until(a,b):
		if not isinstance(b,list): b = [b]
		for i,x in enumerate(a):
			if x == b[0] and ArHelper.same(a[i:],b):
				return a[i + len(b):]
					
	@staticmethod
	def strip_property(a,prop):
		for i,x in enumerate(a):
			if x == prop:
				rv = a[i+1]
				a = a[:i] + a[i+2:]
				return (a,rv)


from jivi.fs import *

class Zoeker:
	def __init__(self,needles=[],files=[]):
		self.needles = needles
		self.files = files
		
		
	def start(self):
		self.needles = [s.lower() for s in self.needles]
		
		self.files_split = list(Ar.steps(self.files,50))
		self.todo = len(self.files_split)
		self.files_found = [{} for i in range(self.todo)]
		
		for i in range(self.todo):
			Thread(self.do,(i,))
			
		while self.todo:
			sleep(1)
			
		found = {}
		for a in self.files_found:
			found.update(a)
		self.files_found = sorted(list(found.keys()))
		
		self.stop()
		
		
	def stop(self):
		print("\n".join(self.files_found))
		
	def check_file(self,fp):
		s = File.all(fp)
		if not s: return 0
		s = s.lower()
		
		for n in self.needles:
			if s.find(n) != -1: return 1
		
			
	def do(self,i):
		for fp in self.files_split[i]:
			fp = File.fp(fp).lower()
			if self.check_file(fp):
				self.files_found[i][fp] = 1
			
		self.todo = self.todo - 1
		
		
	def scanDirs(self,dir,threads=20,**t):

		t['deep'] = 0
		t['full'] = 1
		
		def scanDir(i):
			for fp in self.dirs_to_scan[i]:
				fp = File.fp(fp).lower()
				
				self.files_found[i] += list(Dir.files(fp,**t))
				
			self.todo = self.todo - 1
		
		self.dirs_to_scan = list(Ar.steps(list(Dir.dirs(dir,full=1)),threads))
		self.todo = len(self.dirs_to_scan)
		self.files_found = [[] for i in range(self.todo)]
		
		for i in range(self.todo):
			Thread(scanDir,(i,))
			
		while self.todo:
			sleep(1)
			
		found = []
		for a in self.files_found:
			found += a
		self.files_found = sorted(list(set(found)))
		
	def zoek(self,needles=[]):
		
result = zoeker.zoek(needles=['incompleet'])


from jivi.oe.Statement.Define.Widget import Widget
from jivi.oe.Statement.helpers import *

class Field(Widget):
	type = 'field'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)

	def pre_init(self):
		Widget.pre_init(self)
		self.set_variable()
		


from jivi.oe.Statement.Define.Widget import Widget
from jivi.oe.Statement.helpers import *


to_html_button_format = """<button name = "{name}" style="position: absolute; left: {left}px; top: {top}px;height: {height}px; width: {width}px;" title = "{label}"></button>"""
class Button(Widget):
	type = 'button'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)

				
	def html(self,textra={},war=[]):
		t = {}
		tab = {k : v for k,v in self.tab.items()}
		tab.update(textra)
		if 'label' in tab: tab['label'] = nice_string(tab['label'])
		for a in 'label,left,top,height,width,name'.split(','):
			t[a] = tab[a] if a in tab else ''
		return to_html_button_format.format(**t)


from jivi.oe.constants import *

INPUT_KEYWORD['view-as'] = 1


class Statement:
	maintype = 'statement'
	type = 'statement'
	def __init__(self,ar=[],window=0):
		self.window = window
		self.env = 0
		if window:
			self.env = self.window.env
		
		
		self.ar = ar
		self.ori = ar[:]
		
		self.tab = {}
		self.pre_init()
		
		
		self.read_images()

		for a in INPUT_KEYWORD.keys():
			self.read_input(a)
			
		self.read_size()
		
	
	def html(self,textra={},war=[]):
		return ''
	def read_input(self,name,keyword=0):
		if not keyword:
			keyword = name
		
		if not Keyword(keyword):
			print(keyword)
			exit()
		keyword = Keyword(keyword)
		for i,a in enumerate(self.ar[:-1]):
			if Keyword(a) == keyword:
				self.set(name,Alias(self.ar[i+1]))
				self.remove(i,i+1)
				return 1
				
	def pre_init(self):
		pass
			
	def read_size(self):


		for i,x in enumerate(self.ar):
			kw = Keyword(x)
			if kw.find('size') != -1:
				if len(self.ar) < i + 3:
					self.Error('woops')
					print(self.ori)
				self.set('width',wtp(self.ar[i+1],kw.find('pix') != -1))
				self.set('height',htp(self.ar[i+3],kw.find('pix') != -1))
				return self.remove(i,i+1,i+2,i+3)
	def read_images(self):
		read_images_tab = {'image' : 'image','image-up' : 'image','image-down' : 'image-down','image-insensitive' : 'image-insensitive'}
		
		def read_one():
			lar = len(self.ar)
			for i,x in enumerate(self.ar):
				mainkw = Keyword(x)
				if mainkw in read_images_tab:
					to_remove = [i]
					i += 1
					t = {'file' : 0,'from' : []}
					
				
					def do_next(do_remove=1,forced=0):
						nonlocal i,to_remove
						if forced or i == lar: return (mainkw,t,to_remove)
						i += 1
						if i == lar: return (mainkw,t,to_remove)
						if do_remove: to_remove.append(i)
					
					kw = Keyword(self.ar[i])
					if kw == 'file':
						to_remove.append(i)
						if do_next(): return do_next()
						t['file'] = self.ar[i]
						if do_next(0): return do_next()
						kw = Keyword(self.ar[i])
					
					if kw.find('size') != -1:
						to_remove.append(i)
						if do_next(): return do_next()
						t['w'] = wtp(self.ar[i],kw.find('pix') != -1)
						if do_next(): return do_next()
						t['h'] = htp(self.ar[i],kw.find('pix') != -1)
						if do_next(0): return do_next()
						kw = Keyword(self.ar[i])
						
					if kw == 'from':
						to_remove.append(i)
						if do_next(): return do_next()
						t['from'] = self.ar[i]

					
					return do_next(0,1)
			return (0,0,0)
		mainkw,t,to_remove = read_one()

		while mainkw:
			self.remove(*to_remove)
			if not t['from']:
				del t['from']
			self.set(mainkw,t)
			mainkw,t,to_remove = read_one()
	def set(self,k,v):
		self.tab[k] = v
		setattr(self,k,v)
	def set_if_none(self,k,v):
		if k in self.tab and self.tab[k]: return
		self.set(k,v)
		
	def remove(self,*a):
		ar = sorted(a,reverse=1)
		for i in ar:
			self.ar = self.ar[:i] + self.ar[i+1:]
		
		
	def remove_keyword(self,kw):
		for i,a in enumerate(self.ar):
			if Keyword(a) == Keyword(kw):
				self.remove(i)
				break
	def update(self,t):
		for a,b in t.items():
			self.set(a,b)
			
	def Error(self,s):
		print('-----------------')
		name = self.name if hasattr(self,'name') else "unknown"
		try:
			print(self.window.fp)
		except:
			pass
		print("{type} [{name}] : {s}".format(type=self.type,name=name,s=s))
		print('-----------------')


from jivi.oe.Statement.Statement import Statement


class Define(Statement):
	maintype = 'define'
	
	def __init__(self,*a,**b):
		self.view_as = 0
		
		Statement.__init__(self,*a,**b)
		
		
	def pre_init(self):
		self.name = "unknown"
		self.remove_keyword('define')
		self.remove_keyword('no-undo')
		self.read_input('name',self.type)
		self.set('name',self.name.lower())


from jivi.oe.Statement.Define.Widget import Widget
from jivi.oe.Statement.helpers import *
from json import dumps

from jivi.fs import *
to_html_fill_in_format = """<input class = 'inp' type = "text" style="position: absolute; left: {left}px; top: {top}px;height: {height}px; width: {width}px;" value = "{label}">"""

format_label = """<label class = 'hidden lbl' for="{name}">{label}</label>"""

format_withlabel = """
<div style="position: absolute; left: {left}px; top: {top}px" aligned = "{aligned}">
  {label}
  {input}
</div>
"""


format_input = """<input class = 'inp' id = "{name}" type = "text" style="height: {height}px; width: {width}px;" value = "{initial}">"""

def to_html_fill_in(ob,tab):
	pass

html_tab = {}
html_tab['fill-in'] = to_html_fill_in


class Variable(Widget):
	type = 'variable'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)

	def pre_init(self):
		Widget.pre_init(self)
		self.set_variable()
	def html(self,textra={},war=[]):

		tab = {k : v for k,v in self.tab.items()}
		tab.update(textra)
		if 'label' in tab: tab['label'] = nice_string(tab['label'])
		for a in ['view-as','datatype']:
			if not a in tab:
				File.jwrite('tab.json',tab)
				File.jwrite('ar.json',self.ar)
				File.jwrite('ori.json',self.ori)
				
				
				
				self.Error(a)
				exit()
				
		if tab['view-as'] not in ['fill-in','combo-box','radio-set','toggle-box','editor','text']:
			print(tab['view-as'])
			exit()
		t = create_tab('left,top'.split(','),tab)
		t['input'] = format_input.format(**create_tab('name,height,width,initial'.split(','),tab))
		t['label'] = format_label.format(**create_tab('name,label'.split(','),tab))
		
		t['aligned'] = 'bluh'
		if 'aligned' in tab: t['aligned'] = tab['aligned']
		return format_withlabel.format(**t)


from jivi.oe import *
from jivi.fs import *
from jivi.oe.ArHelper import ArHelper
from jivi.oe.ArHelper import StatementReader as SR

from jivi.oe.Statement.definer import statement
from jivi.oe.Statement.Define.Field import Field

start_str = """
<html>
	<head>
		<script>if (typeof module === 'object') {window.module = module; module = undefined;}</script>
		<script type="text/javascript" src="src/js/jquery.js"></script>
		<script type="text/javascript" src="src/js/lib.js"></script>
		<script type="text/javascript" src="src/js/js.js"></script>
		<script type="text/javascript" src="src/js/prog.js"></script>
		<link rel="stylesheet" type="text/css" href="src/css/lib.css">
		<script>if (window.module) module = window.module;</script>
	</head>
	
	<body>
	"""
	
end_str = """</body></html>"""

class Window:
	steps = ['set_oeReader','set_window','set_defines','write']
	def __init__(self,env,fp):
		self.env = env
		self.fp = File.fp(fp).lower()
		self.window = {}
		self.frames = {}
		self.widgets = {}
		self.temptables = {}
		self.fp_out = "out\\" + File.name(self.fp) + ".html"
		if not File.exists(self.fp_out):
				
			for a in Window.steps:
				if not getattr(self,a)():
					print(self.fp)
					print('failed ' + a)
					break
		
	def write(self):
		for frame in self.frames.values():
			File.write(self.fp_out,start_str + frame.html() + end_str)
			break
		return 1
		
	def get_table(self,tn):
		if tn in self.temptables:
			return self.temptables[tn]
		return self.env.get_table(tn)
	
	def get_field_widget(self,tn,fn,noerror=0):
		table = self.get_table(tn)
		if not table:
			if noerror: return 0
			self.Error('table not found ' + tn)
			return 0
		return table.get_field(fn)
		
		
	def get_widget(self,oriname,noerror=0):


		name = oriname.lower().strip()
		if name in self.defines:
			return self.defines[name]
			
		ar = name.split('.')
		if len(ar) == 2:
			return self.get_field_widget(tn=ar[0],fn=ar[1],noerror=noerror)
		
		if noerror: return 0
		self.Error('widget not found ' + oriname)
		

	
	def set_defines(self):
		self.defines = {}
		self.frames = {}
		for a in self.commands:
			if not a: continue
			if Keyword(a[0]) == 'define':
				d = statement(a,self)
				if d.type == 'frame':
					if d.name in self.frames:
						self.frames[d.name].widget_ar += d.widget_ar
						self.frames[d.name].update(d.tab)
						
					else:
						self.frames[d.name] = d
				elif d.type == "temp-table":
					self.temptables[d.name.lower().strip()] = d
				elif d.name not in self.defines:
					self.defines[d.name] = d
					
				continue
				
		return 1
	def set_oeReader(self):
		self.oeReader = Stripper(self.fp)
		self.commands = self.oeReader.commands
		
			
		return 1
		
	def set_window(self):
		found = self.strip_command(['create','window'],1)
		if not found: return 0
		self.window_index = self.strip_command_last_found_index
		found = ArHelper.strip_until(found,'assign')
		if not found: return 0
		self.window = SR.assign(found)
		return self.window
			
	
	
			
	def strip_command(self,ar,fullblock=1):
		for i,c in enumerate(self.commands):
			for j,cl in enumerate(c):
				if cl == ar[0] and ArHelper.same(c[j:],ar):
					ret_val = c[j:]
					if fullblock:
						self.commands = self.commands[:i] + self.commands[i+1 :]
					else:
						self.commands[i] = self.commands[i][:j] + self.commands[i][j+len(ar):]
					self.strip_command_last_found_index = i
					return ret_val
		

	def Error(self,s):
		print('-----------------')
		print("Window [{name}] : {s}".format(name=self.fp,s=s))
		print('-----------------')


from jivi.fs import *
import configparser
import os
import sys
import win32com.client
from jivi.oe.Table import Table,Field

def jread(fp,encoding='utf-8'):
	with open(fp,"r",encoding=encoding) as f:
		s = f.read()
		a = json.loads(s)
		return a
"""
wd
arg
propath
"""



shortcut_link = r"C:\Users\jorisv\Desktop\Intex_links\local\MEC.lnk"


class Environment:
	def __init__(self,fp=shortcut_link,file_types=['ininame','pf'],sections=['Startup','Colors','fonts'],temp_dir="C:\\temp\\"):
		self.file_types = file_types
		self.sections = sections
		self.temp_dir = temp_dir
		shell = win32com.client.Dispatch("WScript.Shell")
		self.shortcut = shell.CreateShortCut(fp)
		self.path = self.shortcut.Targetpath
		self.wd = self.shortcut.WorkingDirectory
		self.arg = {}
		self.propath = []
		self.set_arg()
		self.set_sections()
		self.set_fields_and_tables()
		self.set_files()

	def get_field(self,fn):
		fn = fn.lower()
		return self.fields[fn] if fn in self.fields else 0

	def get_table(self,fn):
		fn = fn.lower()
		return self.tables[fn] if fn in self.tables else 0

	def get_table_name(self,fn):
		fn = fn.lower()
		return fn if fn in self.tables else 0

	def get_field_widget(self,fn):
		return self.get_field(fn)

	def set_arg(self):
		arg = [a for a in [a.strip() for a in self.shortcut.Arguments.split(' ')] if a]
		meep = []
		key = 0
		for a in arg:
			if a.startswith('-'):
				if key:
					self.arg[key] = ' '.join(meep)
					meep = []
				key = a[1:]
			elif key:
				meep.append(a)
		if key:
			self.arg[key] = ' '.join(meep)
		for k,v in self.arg.items():
			if not hasattr(self,k):
				setattr(self,k,v)
		for a in self.file_types:
			if a in self.arg:
				if not os.path.isfile(self.arg[a]):
					self.arg[a] = os.path.join(self.wd,self.arg[a])

	def get_fp(self,rel_fp):
		rel_fp = rel_fp.lower().strip().replace("/","\\")
		return self.files_pp[rel_fp]

	def len_chars(self,s,font):
		if not font in self.font_lenght:
			print("Environment : font not found {font}".format(font=font))
			return 0
		font = self.font_lenght[font]
		l = 0
		for a in s:
			if not a in font:
				print("Environment len_chars {a} not found!".format(a=a))
				continue
			l += font[a][1]
		return l

	def len_pixels(self,s,font):
		if not font in self.font_lenght:
			print("Environment : font not found {font}".format(font=font))
			return 0
		font = self.font_lenght[font]
		l = 0
		for a in s:
			if not a in font:
				print("Environment get_length_pixels {a} not found!".format(a=a))
				continue
			l += font[a][0]
		return l

	def set_fields_and_tables(self):
		new_args = {k : v for k,v in self.arg.items()}
		new_args['p'] = r"C:\prog\jv\py\dbstruct.p"
		new_args = ["-{k} {v}".format(k=k,v=v) for k,v in new_args.items()]
		#os.system("{path} {new_args}".format(path=self.path,new_args=" ".join(new_args)))
		tabs = {}
		for a in ['ttField','ttIndex','ttIndexField','ttFile','ttFont']:
			tabs[a[2:].lower()] = jread(r"C:\temp\{a}.json".format(a=a))[a]
		self.fields = {}
		self.tables = {}
		self.indexes = {}
		file_by_recid = {}
		self.font_lenght = {}
		
		
		
		#fi cc cl pl
		for a in tabs['font']:
			fi = a['fi']
			cc = a['cc']
			cl = a['cl']
			pl = a['pl']
			if not fi in self.font_lenght:
				self.font_lenght[fi] = {}
			self.font_lenght[fi][cc] = [pl,cl]

		def get_key(k,type):
			k = k.lstrip('_').replace('-','_').lower().strip()
			if k.startswith(type):
				k = k[len(type) + 1:]
			return k

		def check_tab(tab,required,defaults):
			for k,v in defaults.items():
				if not k in tab:
					tab[k] = v
			for a in required:
				if not a in tab: return 0
			return tab
		for t in tabs['file']:
			tab = {get_key(k,'file') : v for k,v in t.items()}
			tab = check_tab(tab,['rrecid','name'],{})
			if not tab: continue
			tab['name'] = tab['name'].lower()
			table = Table().from_json(tab)
			self.tables[table.name] = table
			file_by_recid[tab['rrecid']] = table
		for t in tabs['field']:
			tab = {get_key(k,'field') : v for k,v in t.items()}
			tab = check_tab(tab,['rrecid','name','file_recid'],dict(label=0,format=0))
			if not tab: continue
			if not 'name' in tab or (not tab['name']): continue
			tab['name'] = tab['name'].lower()
			table = file_by_recid[tab['file_recid']] if tab['file_recid'] in file_by_recid else 0
			if not table: continue
			tab['table'] = table
			field = Field().from_json(tab)
			self.fields[field.fullName] = tab
		
		
		
		
		
		self.alias_table = {}
		bad = {}
		
		for tablename,table in self.tables.items():
			table.create_alias_fields()
			for i in range(len(tablename) - 1,1,-1):
				tmp = tablename[:i]
				if tmp in self.alias_table:
					bad[tmp] = 1
					del self.alias_table[tmp]
				if tmp in bad: continue
				self.alias_table[tmp] = table
				
				
		
				
	def set_files(self):
		self.files = []
		tab = {}
		self.files_pp = {}
		for d in self.propath:
			for fp in Dir.files(d,ex=['w','p','i'],full=1,deep=1):
				fp = fp.lower()
				tab[fp] = 1
		for fp in tab.keys():
			pp = []
			for p in self.propath:
				if fp.startswith(p):
					file_pp = fp[len(p):]
					if not file_pp in self.files_pp:
						self.files_pp[file_pp] = fp
					pp.append(file_pp)
			self.files.append(dict(fp=fp,pp=pp,ex=File.ex(fp).lower()))

	def fp(self,fp):
		if os.path.isfile(fp):
			return os.path.abspath(fp)
		for d in self.propath:
			x = os.path.join(d,fp)
			if os.path.isfile(x):
				return os.path.abspath(x)
		return 0

	def fp_dir(self,fp):
		if not os.path.isdir(fp):
			fp = os.path.join(self.wd,fp)
		if not os.path.isdir(fp):
			return 0
		return os.path.abspath(fp).rstrip(os.sep) + os.sep

	def set_sections(self):
		config = configparser.ConfigParser(strict=0)
		config.read(self.arg['ininame'])
		for a in self.sections:
			t = {}
			sn = a.lower()
			if a in config:
				ca = config[a]
				for x in ca:
					t[x.lower()] = ca[x]
			setattr(self,sn,t)
		for a in self.startup['propath'].split(','):
			fp = self.fp_dir(a.strip())
			if fp:
				self.propath.append(fp.lower())


import json,re,os,math,subprocess,sys
from time import time,sleep
from datetime import datetime,timedelta
from _thread import start_new_thread as Thread
from math import ceil


from datetime import datetime,timedelta

try:
	import pytz
	
except:
	pass



pabs = os.path.abspath
def print_keys(t):
	print("\n".join(list(t.keys())))
def pjoin(*args): return pabs(os.path.join(*args))

DEFAULT_ENCODING = 0
def Decode(b):
	for a in [DEFAULT_ENCODING] + ['cp1252','ascii','utf-8','latin-1']:
		try:
			s = b.decode(a)
			return s
		except:
			pass
	
def Popen(s):
	try:
		process = subprocess.Popen(s, stdout=subprocess.PIPE,shell=1)
		stdout, stderr = process.communicate()
		if stderr: print(stderr)
		
		stdout = Decode(stdout)
		for x in stdout.splitlines():
			yield x.strip()
	except:
		pass
	

DEFAULT_ENCODING = "cp850" #'cp' + list(Popen('chcp'))[0].split(' ')[-1]

def Command(cmd):
	process = subprocess.Popen(cmd, shell = True)
	stdout, stderr = process.communicate()
	

	
def Pause():
	os.system('pause')
	
	
def input_yes_no(question, default="yes"):
	valid = {"yes": True, "y": True, "ye": True,"no": False, "n": False}
	if default is None:
		prompt = " [y/n] "
	elif default == "yes":
		prompt = " [Y/n] "
	elif default == "no":
		prompt = " [y/N] "
	else:
		raise ValueError("invalid default answer: '%s'" % default)

	while True:
		sys.stdout.write(question + prompt)
		choice = input().lower()
		if default is not None and choice == '':
			return valid[default]
		elif choice in valid:
			return valid[choice]
		else:
			sys.stdout.write("Please respond with 'yes' or 'no' (or 'y' or 'n').\n")
							 
							 


class Data:
	_standard = {"__class__": 1, "__delattr__": 1, "__dict__": 1, "__dir__": 1, "__doc__": 1, "__eq__": 1, "__format__": 1, "__ge__": 1, "__getattribute__": 1, "__gt__": 1, "__hash__": 1, "__init__": 1, "__init_subclass__": 1, "__le__": 1, "__lt__": 1, "__module__": 1, "__ne__": 1, "__new__": 1, "__reduce__": 1, "__reduce_ex__": 1, "__repr__": 1, "__setattr__": 1, "__sizeof__": 1, "__str__": 1, "__subclasshook__": 1, "__weakref__": 1, "_set": 1, "_standard": 1, "items": 1}
	
	def __init__(self,**t):
		self._set(**t)
	def _set(self,**t):
		for a,b in t.items(): setattr(self,a,b)
	def items(self):
		for a in dir(self):
			if not a in self._standard:
				yield a,getattr(self,a)
				



class Time:
	breaks = 0
	def __init__(self,name=None,func=None,amount=1000):
		self.name = name + " - " if name else ""
		self.start = time()
		self.stop_time = 0
		if func:
			for i in range(amount):
				func()
			self.stop()
		
				
	def stop(self):
		self.stop_time = time()
		print(self.passed)
		
				
	def tussenstop(self,n=""):
		self.breaks += 1
		print(self.passed,"[%d %s]" % (self.breaks,n))
	
	
	@property
	def passed(self,x=""):
		d = self.stop_time if self.stop_time else time()
		d -= self.start
		return self.name + x + str(timedelta(seconds=d))


		





			
			
			
class Ar:
	@staticmethod
	def split(ar,c):
		for i in range(0, len(ar), c):
			yield ar[i:i + c]
	@staticmethod
	def steps(ar,steps=10):
		c = ceil(len(ar)/steps)
		if c == 0: return
		for i in range(0, len(ar), c):
			yield ar[i:i + c]
	
	@staticmethod
	def filter(ar,strip=1):
		for x in ar:
			x = str(x).strip()
			if x:
				yield x
		


from jivi.oe.Statement.helpers import *

from jivi.oe.Statement.Define.all import *


define_tab = {k.lower() : v for k,v in dict(Dataset=Dataset,stream=Stream,Workfile=Workfile,Query=Query,Menu=Menu,Parameter=Parameter,Frame=Frame,Button=Button,Browse=Browse,Buffer=Buffer,Image=Image,Rectangle=Rectangle,Variable=Variable).items()}
define_tab['temp-table'] = Temptable
define_tab['sub-menu'] = Submenu
define_tab['work-table'] = Worktable




not_implented = {}

def define(ar,window):
	global not_implented
	for i,a in enumerate(ar):
		type = DefineType(a)
		if not type: continue
	
		if not type in define_tab:
			not_implented[type] = 1
		else:
			return define_tab[type](ar,window)
		break

statements = {}
statements['define'] = define

		

def statement(ar,window):
	if not ar: return 0
	kw = Keyword(ar[0])
	if not kw:
		print("statement start is not a keyword {start}".format(start=ar[0]))
		return 0
	if not kw in statements:
		print("statement keyword {kw} not yet implemented".format(kw=kw))
		return 0
		
	return statements[kw](ar,window)


from jivi.oe.Statement.Define.Widget import Widget
class Menu(Widget):
	type = 'menu'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)




class Submenu(Widget):
	type = 'sub-menu'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)


from jivi.core import *
import math
try:
	from django.utils import timezone
except:
	pass

class Dir:
	@staticmethod
	def up(fp,n=1):
		return Dir.join(fp,('..' + os.sep)*n)

	@staticmethod
	def name(fp):
		return os.path.basename(Dir.fp(fp).rstrip(os.sep))
	
	@staticmethod
	def rel(fp,b):
		return os.path.relpath(fp,b).rstrip(os.sep)
	
	@staticmethod
	def find(fp,fn,deep=1):
		flags = '/b/a-d'
		if deep: flags += '/s'
		fp = Dir.fp(fp)
		s = 'dir {fp}{fn} {flags} 2>nul'.format(fp=Dir.fp(fp),fn=fn,flags=flags)
		for x in Popen(s): yield x
	@staticmethod
	def files(dir,full=1,ex=None,deep=None,img=None):
		if img:
			ex = ['bmp','jpg','jpeg','gif','png']
		dir = Dir.fp(dir)
		flags = '/b/a-d'
		if deep: 
			flags += '/s'
			full = 0
		if ex:
			if isinstance(ex,str): ex = [ex]
		else:
			ex = ["*"]
		s = 'dir {x} {flags} 2>nul'.format(x=" ".join(['"{dir}*.{e}"'.format(dir=dir,e=e) for e in ex]),flags=flags)
		
		for x in Popen(s):
			yield dir + x if full else x
	@staticmethod
	def dirs(dir,full=1,deep=None):
		dir = Dir.fp(dir)
		flags = '/b/ad'
		if deep: 
			flags += '/s'
			full = 0
		s = 'dir "{dir}" {flags} 2>nul'.format(dir=dir,flags=flags)
		for x in Popen(s):
			yield dir + x if full else x
	@staticmethod
	def exists(fp):
		return os.path.isdir(fp)
	@staticmethod
	def join(fp,d=0):
		return Dir.fp(os.path.join(fp,d) if d else fp)
	@staticmethod
	def fp(fp,d=0):
		fp = os.path.join(d,fp) if d else fp
		return os.path.abspath(fp).rstrip(os.sep) + os.sep
	@staticmethod
	def create(fp):
		fp = Dir.fp(fp)
		Command('IF NOT EXIST "{fp}" mkdir "{fp}"'.format(fp=fp))
		return fp
		
class File:
	@staticmethod
	def base(fp):
		return os.path.basename(fp)
	@staticmethod
	def fp(fp,dir=0):
		if dir:
			fp = os.path.join(dir,fp)
		return os.path.abspath(fp)
	@staticmethod
	def modified(fp,dt=False):
		try:
			m =os.path.getmtime(fp)
			if dt:
				return datetime.fromtimestamp(m, tz=timezone.utc)
			return m
		except:
			return 0
	@staticmethod
	def write(fp,s):
		try:
			with open(fp,'w') as f:
				f.write(s)
		except:
			return None
	@staticmethod
	def arwrite(fp,ar):
		try:
			with open(fp,'w') as f:
				f.write("\n".join(ar))
		except:
			return None
	@staticmethod
	def jread(fp,encoding='cp1252'):
		with open(fp,"r",encoding=encoding) as f:
			s = f.read()
			a = json.loads(s)
			return a
	@staticmethod
	def jwrite(fp,ob,indent=4,carefull=0):
		fp = File.fp(fp)
		
		if carefull:
			fp_tmp = File.dir(fp) + File.name(fp) + str(time()) + '.tmp'
			with open(fp_tmp,'w') as f:
				json.dump(ob,f,indent=indent)
			if File.jread(fp_tmp) != None:
				shutil.move(fp_tmp, fp)
				return True
			else:
				File.delete(fp)
		else:
			with open(fp,'w') as f:
				json.dump(ob,f,indent=indent)
	@staticmethod
	def delete(fp):
		try:
			os.remove(fp)
		except:
			return None
	@staticmethod
	def ex(fp,lower=True):
		if lower: return File.ex(fp,lower=False).lower()
		try:
			return os.path.splitext(fp)[1][1:]
		except:
			return ""
	@staticmethod
	def dir(fp):
		return Dir.fp(os.path.dirname(fp))
	@staticmethod
	def name(fp,lower=False):
		if lower: return File.name(fp).lower()
		return os.path.splitext(os.path.basename(fp))[0]
	@staticmethod
	def all(fp):
		try:
			with open(fp,'r') as f:
				x = f.read()
				return x
		except:
			pass
	@staticmethod
	def lines(fp,strip=1):
		try:
			with open(fp,'r') as f:
				x = [a.rstrip() if strip else a for a in f.readlines()]
				return x
		except:
			pass
		return []
	@staticmethod
	def lines_specialx(fp):
		try:
			with open(fp,'r') as f:
				x = []
				for l in f.readlines():
					line = l.rstrip()
					if not line:
						line = " "
					x.append(line)
				
				return x
		except:
			pass
	@staticmethod
	def lines_special(fp):
		
		try:
			x = File.all(fp)
			x = x.splitlines()
			return x
		except:
			pass
	@staticmethod
	def exists(fp):
		return os.path.isfile(fp)
		
	
	
def write_ar(fp,ar,name='ar',type='char'):
	l = len(ar)
	vars = json.dumps(ar)
	
	with open(fp,'w') as f:
		f.write("def var {name} as {type} extent {l} init {vars} no-undo.".format(name=name,type=type,l=l,vars=vars))





class String:
	@staticmethod
	def tabs(s,length):
		
		tabs = math.ceil(length/4)
		tabs_string = math.floor(len(s)/4)
		s = "\t"*max(0,tabs - tabs_string)
		
		return s
	
	@staticmethod
	def tabbed(s,length):
		return s + String.tabs(s,length)
	
		
	@staticmethod
	def tab(tab):
		keys = list(tab.keys())
		max_len = len(max(keys,key=len))
		
		return "\n".join(["{k} : {v}".format(k=String.tabbed(k,max_len),v=tab[k]) for k in keys])




from jivi.oe.Statement.Define.Widget import Widget

class Table:
	def __init__(self):
		self.fields = {}
		self.name = "not set"
		
	def from_json(self,t):
		
		for a,b in t.items():
			setattr(self,a,b)
			
		self.name = self.name.lower().strip()
		return self
		
		
	def get_field(self,fn):
		if not fn in self.fields:
			self.Error('field not found' + fn)
		return self.fields[fn]
	
	def Error(self,s):
		print("Table {name} : {s}".format(name=self.name,s=s))
	
	
	def create_alias_fields(self):
		self.alias_field = {}
		bad = {}
		
		for fieldname,field in self.fields.items():
			for i in range(len(fieldname) - 1,1,-1):
				tmp = fieldname[:i]
				if tmp in self.alias_field:
					bad[tmp] = 1
					del self.alias_field[tmp]
				if tmp in bad: continue
				self.alias_field[tmp] = table

class Field(Widget):
	type = 'field'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)
		self.fullName = ''
		self.name = ''

	def from_json(self,t):
		conv = [('data_type','datatype'),('view_as','view-as')]
		for van,naar in conv:
			
			t[naar] = t[van]
			del t[van]
			
		
		for a,b in t.items():
			self.set(a,b)
			
		self.fullName = self.table.name + '.' + self.name
		
		self.fullName = self.fullName.lower().strip()
		self.name = self.name.lower().strip()
		
		self.table.fields[self.name] = self

		return self



from jivi.oe.Statement.Define.Define import Define
from jivi.oe.Statement.Define.Field import Field

class Temptable(Define):
	type = 'temp-table'
	def __init__(self,*a,**b):
		self.fields = {}
		Define.__init__(self,*a,**b)
		
		
	def pre_init(self):
		Define.pre_init(self)
		to_remove = []
		self.header = []
		
		for ind,a in enumerate(self.ar):
			if a.lower().strip() in ['index','field']:
				break
			to_remove.append(ind)
			self.header.append(a)
		self.remove(*to_remove)
		to_remove = []
		
		tmpvelden = []
		veld = []
		velden = []
		for ind,a in enumerate(self.ar):
			n = a.lower().strip()
			if n == 'index':
				break
			to_remove.append(ind)
			if n == 'field':
				if veld:
					tmpvelden.append(veld)
					velden.append(Field(veld,self.window))
				veld = []
			veld.append(n)
			
		if veld:
			velden.append(Field(veld,self.window))
			tmpvelden.append(veld)
			
		self.tmpvelden = tmpvelden
		for veld in velden:
			self.fields[veld.name.lower()] = veld
		self.remove(*to_remove)
		
		
	def get_field(self,fn):
		if not fn in self.fields:
			print(self.tmpvelden)
			self.Error('field not found' + fn + "\n" + "\n".join(list(self.fields.keys())))
			
			
		return self.fields[fn]
	



from jivi.oe.Statement.Define.Widget import Widget
from jivi.oe.Statement.helpers import *

class Frame(Widget):
	type = 'frame'
	def __init__(self,*a,**b):
		Widget.__init__(self,*a,**b)
		self.widgets = 0
			
	def pre_init(self):
		Widget.pre_init(self)
		i = len(self.ar) - 1
		with_ar = []
		while i != -1:
			if self.ar[i] == 'with':
				with_ar = self.ar[i+1:]
				self.ar = self.ar[:i]
				break
			i -= 1
			
		self.widget_ar = self.ar[:]
		self.ar = with_ar[:]
		

		
	def set_widgets(self,wids):
		widgets = []
		
		war = []
		widget = 0
		
		for i,c in enumerate(self.widget_ar):
			wid = self.window.get_widget(c,1)
			if wid:
				if war:
					widgets.append((widget,war))
				war = []
				widget = wid
			elif is_string(c):
				
				if (not i) or ((((i + 1) < len(self.widget_ar)) and (Keyword(self.widget_ar[i+1]) == 'view-as')) and (not Keyword(self.widget_ar[i-1]) in INPUT_KEYWORD)):
					if war:
						widgets.append((widget,war))
					widget = 0
					war = []
			
			if c: war.append(c)
		
		if war:
			widgets.append((widget,war))
		
		self.read_widgets(widgets)
		
		
	def read_widgets(self,widgets):
		self.widgets = []
		def read_pos(war):
			for i,a in enumerate(war):
				if Keyword(a) == 'at':
					to_remove = [i,i+1,i+2,i+3,i+4]
					top_type = Keyword(war[i+1])
					top_val = war[i+2]
					
					
					
					left_type = Keyword(war[i+3])
					left_val = war[i+4]
					t = dict(top=rtp(top_val,top_type == 'y'),left=ctp(left_val,left_type == 'x'),aligned='normal')
					if t['top'] is None: self.Error("read_pos top is None " + top_val)
					if t['left'] is None: 
						self.Error("read_pos left is None " + left_val)
						t['left'] = 0
						
					
					if ((i + 5) < len(war)) and Keyword(war[i+5]).find('aligned') != -1:
						t['aligned'] = Keyword(war[i+5])
						to_remove.append(i+5)
			
					if t['aligned'] == 'colon-aligned':
						t['left'] += ( 2 * pixels_per_col )
					
					return (t,remove_from_list(war,to_remove))
					
			return ({},war)

		for widget,war in widgets:
			t,war = read_pos(war)
			if not t:
				self.Error("position not found " + str(war))
				continue
			self.widgets.append((t,widget,war))
		
	def html(self,textra={},war=[]):
		if self.widgets == 0:
			self.set_widgets(self.window.defines)
		kak = []
		to_html_frame = """<div class = "frame" style="position: relative; width : 100%; height : 100%">{inner}</div>"""
		
		
		for textra,widget,war in self.widgets:
			if widget:
				kak.append(widget.html(textra,war))
				
		return to_html_frame.format(inner="\n".join(kak))


from jivi.oe.Statement.Define.Browse import Browse
from jivi.oe.Statement.Define.Buffer import Buffer
from jivi.oe.Statement.Define.Button import Button
from jivi.oe.Statement.Define.Dataset import Dataset
from jivi.oe.Statement.Define.Define import Define
from jivi.oe.Statement.Define.Field import Field
from jivi.oe.Statement.Define.Frame import Frame
from jivi.oe.Statement.Define.Image import Image
from jivi.oe.Statement.Define.Menu import Menu,Submenu
from jivi.oe.Statement.Define.Parameter import Parameter
from jivi.oe.Statement.Define.Query import Query
from jivi.oe.Statement.Define.Rectangle import Rectangle
from jivi.oe.Statement.Define.Stream import Stream
from jivi.oe.Statement.Define.Temptable import Temptable
from jivi.oe.Statement.Define.Variable import Variable
from jivi.oe.Statement.Define.Widget import Widget
from jivi.oe.Statement.Define.Workfile import Workfile
from jivi.oe.Statement.Define.Worktable import Worktable

